@@ -17,10 +17,14 @@ ValueProp, ...).
 # pylint: disable-msg=W0622
 # pylint: disable-msg=W0212
 
+from __future__ import print_function, unicode_literals
+
 import sys
 import re
+import collections
 
 from guidata.utils import utf8_to_unicode, update_dataset
+from guidata.py3compat import to_text_string, is_text_string, PY2
 
 
 DEBUG_DESERIALIZE = False
@@ -71,8 +75,8 @@ class FormatProp(ItemProperty):
             return self.fmt % dic
         except TypeError:
             if not self.ignore_error:
-                print "Wrong Format for %s : %r %% %r"\
-                      % (item._name, self.fmt, dic)
+                print("Wrong Format for %s : %r %% %r"\
+                      % (item._name, self.fmt, dic))
                 raise
 
 class GetAttrProp(ItemProperty):
@@ -135,9 +139,9 @@ class DataItem(object):
     """
     DataSet data item
     
-    `label` : (str, unicode)
+    `label` : string
     `default` : any type, optional
-    `help` : (str, unicode) Text displayed on data item's tooltip
+    `help` : string Text displayed on data item's tooltip
     """
     count = 0
 
@@ -200,7 +204,7 @@ class DataItem(object):
         help = self._help
         if auto_help:
             if help:
-                help = help + u"\n(" + auto_help + u")"
+                help = help + "\n(" + auto_help + ")"
             else:
                 help = auto_help.capitalize()
         return help
@@ -209,7 +213,7 @@ class DataItem(object):
         """
         Return the automatically generated part of data item's tooltip
         """
-        return u""
+        return ""
         
     def format_string(self, instance, value, fmt, func):
         """Apply format to string representation of the item's value"""
@@ -225,17 +229,18 @@ class DataItem(object):
         if repval is not None:
             return repval
         else:
-            fmt = self.get_prop_value("display", instance, "format", u"%s")
+            fmt = self.get_prop_value("display", instance, "format", "%s")
             func = self.get_prop_value("display", instance, "func", lambda x:x)
-            if callable(fmt) and value is not None:
+            if isinstance(fmt, collections.Callable) and value is not None:
                 return fmt(func(value))
-            elif isinstance(fmt, basestring):
-                fmt = unicode(fmt)
+            elif is_text_string(fmt):
+                # This is only necessary with Python 2: converting to unicode
+                fmt = to_text_string(fmt)
 
             if value is not None:
                 text = self.format_string(instance, value, fmt, func)
             else:
-                text = u""
+                text = ""
             return text
 
     def set_name(self, new_name):
@@ -336,12 +341,13 @@ class DataItem(object):
         """
         try:
             value = self.get_value_from_reader(reader)
-        except RuntimeError, e:
+        except RuntimeError as e:
             if DEBUG_DESERIALIZE:
                 import traceback
-                print >>sys.stderr, "DEBUG_DESERIALIZE enabled in datatypes.py"
+                print("DEBUG_DESERIALIZE enabled in datatypes.py",
+                      file=sys.stderr)
                 traceback.print_stack()
-                print >>sys.stderr, e
+                print(e, file=sys.stderr)
             self.set_default(instance)
             return
         self.__set__(instance, value)
@@ -554,18 +560,24 @@ class DataSetMeta(type):
                 for item in base._items:
                     items[item._name] = item
                 
-        for attrname, value in dct.items():
+        for attrname, value in list(dct.items()):
             if isinstance( value, DataItem ):
                 value.set_name(attrname)
                 if attrname in items:
                     value._order = items[attrname]._order
                 items[attrname] = value
-        items_list = items.values()
+        items_list = list(items.values())
         items_list.sort(key=lambda x:x._order)
         dct["_items"] = items_list
         return type.__new__(cls, name, bases, dct)
 
-class DataSet(object):
+if PY2:
+    CLS_NAME = b'Meta_Py3Compat'
+else:
+    CLS_NAME = 'Meta_Py3Compat'
+Meta_Py3Compat = DataSetMeta(CLS_NAME, (object, ), {})
+
+class DataSet(Meta_Py3Compat):
     """
     Construct a DataSet object is a set of DataItem objects
         * title [string]
@@ -573,8 +585,8 @@ class DataSet(object):
         * icon [QIcon or string]: icon show on the button (optional)
           (string: icon filename as in guidata/guiqwt image search paths)
     """
-    __metaclass__ = DataSetMeta
-
+    __metaclass__ = DataSetMeta # keep it even with Python 3 (see DataSetMeta)
+    
     def __init__(self, title=None, comment=None, icon=''):
         self.__title = title
         self.__comment = comment
@@ -596,7 +608,7 @@ class DataSet(object):
         cannot be translated at the time they are created.
         """
         module = sys.modules[self.__class__.__module__]
-        if hasattr(module,"_"):
+        if hasattr(module, "_"):
             return module._
         else:
             return lambda x:x
@@ -658,7 +670,7 @@ class DataSet(object):
         """
         Edit data set with text input only
         """
-        import textedit
+        from guidata.dataset import textedit
         self.accept(textedit.TextEditVisitor(self))
 
     def edit(self, parent=None, apply=None, size=None):
@@ -750,13 +762,12 @@ class DataSet(object):
             with reader.group(item._name):
                 try:
                     item.deserialize(self, reader)
-                except RuntimeError, error:
+                except RuntimeError as error:
                     if DEBUG_DESERIALIZE:
                         import traceback
-                        print >>sys.stderr,\
-                            "DEBUG_DESERIALIZE enabled in datatypes.py"
+                        print("DEBUG_DESERIALIZE enabled in datatypes.py", file=sys.stderr)
                         traceback.print_stack()
-                        print >>sys.stderr, error
+                        print(error, file=sys.stderr)
                     item.set_default(self)
             
     def read_config(self, conf, section, option):
