@@ -16,13 +16,17 @@ package distribution on Microsoft Windows platforms with ``py2exe`` or on
 all platforms thanks to ``cx_Freeze``.
 """
 
+from __future__ import print_function
+
 import sys
 import os
 import os.path as osp
 import shutil
 import traceback
 import atexit
+import imp
 from subprocess import Popen, PIPE
+import warnings
 
 # Local imports
 from guidata.configtools import get_module_path
@@ -42,7 +46,6 @@ def get_changeset(path, rev=None):
     except IndexError:
         raise RuntimeError(process.stderr.read())
 
-
 def prepend_module_to_path(module_path):
     """
     Prepend to sys.path module located in *module_path*
@@ -50,8 +53,7 @@ def prepend_module_to_path(module_path):
     
     Use this function:
     1) In your application to import local frozen copies of internal libraries
-    2) In your py2exe distributed package to add a text file containing the 
-       returned string
+    2) In your py2exe distributed package to add a text file containing the returned string
     """
     if not osp.isdir(module_path):
         # Assuming py2exe distribution
@@ -62,7 +64,19 @@ def prepend_module_to_path(module_path):
     prefix = "Prepending module to sys.path"
     message = prefix + ("%s [revision %s]" % (name, changeset)
                         ).rjust(80 - len(prefix), ".")
-    print >> sys.stderr, message
+    print(message, file=sys.stderr)
+    if name in sys.modules:
+        sys.modules.pop(name)
+        nbsp = 0
+        for modname in sys.modules.keys():
+            if modname.startswith(name + '.'):
+                sys.modules.pop(modname)
+                nbsp += 1
+        warning = '(removed %s from sys.modules' % name
+        if nbsp:
+            warning += ' and %d subpackages' % nbsp
+        warning += ')'
+        print(warning.rjust(80), file=sys.stderr)
     return message
 
 def prepend_modules_to_path(module_base_path):
@@ -80,34 +94,129 @@ def prepend_modules_to_path(module_base_path):
 #==============================================================================
 # Distribution helpers
 #==============================================================================
-def create_vs2008_data_files(verbose=False):
-    """Including Microsoft Visual C++ 2008 DLLs"""    
-    filelist = []
-    manifest = osp.join(sys.prefix, "Microsoft.VC90.CRT.manifest")
-    filelist.append(manifest)
+def _remove_later(fname):
+    """Try to remove file later (at exit)"""
+    def try_to_remove(fname):
+        if osp.exists(fname):
+            os.remove(fname)
+    atexit.register(try_to_remove, osp.abspath(fname))
+
+def get_msvc_version(python_version):
+    """Return Microsoft Visual C++ version used to build this Python version"""
+    if python_version is None:
+        python_version = '2.7'
+        warnings.warn("assuming Python 2.7 target")
+    if python_version in ('2.6', '2.7', '3.0', '3.1', '3.2'):
+        # Python 2.6-2.7, 3.0-3.2 were built with Visual Studio 9.0.21022.8
+        # (i.e. Visual C++ 2008, not Visual C++ 2008 SP1!)
+        return "9.0.21022.8"
+    elif python_version in ('3.3', '3.4'):
+        # Python 3.3+ were built with Visual Studio 10.0.30319.1
+        # (i.e. Visual C++ 2010)
+        return '10.0'
+    else:
+        raise RuntimeError("Unsupported Python version %s" % python_version)
+
+def get_msvc_dlls(architecture=None, python_version=None):
+    """Get the list of Microsoft Visual C++ DLLs associated to 
+    architecture and Python version, create the manifest file.
     
-    from xml.etree import ElementTree
-    assembly = ElementTree.fromstring(file(manifest).read())
-    assid = assembly.find("{urn:schemas-microsoft-com:asm.v1}assemblyIdentity")
-    version = assid.get("version")
-    arch = assid.get("processorArchitecture")
-    key = assid.get("publicKeyToken")
+    architecture: integer (32 or 64) -- if None, take the Python build arch
+    python_version: X.Y"""
+    current_architecture = 64 if sys.maxsize > 2**32 else 32
+    if architecture is None:
+        architecture = current_architecture
 
-    vc_str = '%s_Microsoft.VC90.CRT_%s_%s' % (arch, key, version)
-    winsxs = osp.join(os.environ['windir'], 'WinSxS')
-    for fname in os.listdir(winsxs):
-        path = osp.join(winsxs, fname)
-        if osp.isdir(path) and fname.startswith(vc_str):
-            for dllname in os.listdir(path):
-                filelist.append(osp.join(path, dllname))
-            break
+    filelist = []
 
-    print create_vs2008_data_files.__doc__
+    msvc_version = get_msvc_version(python_version)
+    msvc_major = msvc_version.split('.')[0]
+    if msvc_major == '9':
+        key = "1fc8b3b9a1e18e3b"
+        atype = "" if architecture == 64 else "win32"
+        arch = "amd64" if architecture == 64 else "x86"
+        manifest = """<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
+<!-- Copyright (c) Microsoft Corporation.  All rights reserved. -->
+<assembly xmlns="urn:schemas-microsoft-com:asm.v1" manifestVersion="1.0">
+    <noInheritable/>
+    <assemblyIdentity
+        type="%(atype)s"
+        name="Microsoft.VC90.CRT"
+        version="%(version)s"
+        processorArchitecture="%(arch)s"
+        publicKeyToken="%(key)s"
+    />
+    <file name="msvcr90.dll" />
+    <file name="msvcp90.dll" />
+    <file name="msvcm90.dll" />
+</assembly>
+""" % dict(version=msvc_version, key=key, atype=atype, arch=arch)
+
+        vc90man = "Microsoft.VC90.CRT.manifest"
+        open(vc90man, 'w').write(manifest)
+        _remove_later(vc90man)
+        filelist += [vc90man]
+
+        vc_str = '%s_Microsoft.VC90.CRT_%s_%s' % (arch, key, msvc_version)
+        winsxs = osp.join(os.environ['windir'], 'WinSxS')
+        for fname in os.listdir(winsxs):
+            path = osp.join(winsxs, fname)
+            if osp.isdir(path) and fname.lower().startswith(vc_str.lower()):
+                for dllname in os.listdir(path):
+                    filelist.append(osp.join(path, dllname))
+                break
+        else:
+            raise RuntimeError("Microsoft Visual C++ DLLs version %s "\
+                                "were not found" % msvc_version)
+
+    elif msvc_major == '10':
+        namelist = ['msvcp100.dll', 'msvcr100.dll']
+        
+        windir = os.environ['windir']
+        is_64bit_windows = osp.isdir(osp.join(windir, "SysWOW64"))
+
+        # Reminder: WoW64 (*W*indows 32-bit *o*n *W*indows *64*-bit) is a 
+        # subsystem of the Windows operating system capable of running 32-bit 
+        # applications and is included on all 64-bit versions of Windows
+        # (source: http://en.wikipedia.org/wiki/WoW64)
+        #
+        # In other words, "SysWOW64" contains 64-bit DLL and applications, 
+        # whereas "System32" contains 64-bit DLL and applications on a 64-bit 
+        # system.
+        sysdir = "System32"
+        if not is_64bit_windows and architecture == 64:
+            raise RuntimeError("Can't find 64-bit MSVC DLLs on a 32-bit OS")
+        if is_64bit_windows and architecture == 32:
+            sysdir = "SysWOW64"
+
+        for dllname in namelist:
+            fname = osp.join(windir, sysdir, dllname)
+            if osp.exists(fname):
+                filelist.append(fname)
+            else:
+                raise RuntimeError("Microsoft Visual C++ DLLs version %s "\
+                                   "were not found" % msvc_version)
+
+    else:
+        raise RuntimeError("Unsupported MSVC version %s" % msvc_version)
+    
+    return filelist
+
+def create_msvc_data_files(architecture=None, python_version=None,
+                           verbose=False):
+    """Including Microsoft Visual C++ DLLs"""
+    filelist = get_msvc_dlls(architecture=architecture,
+                             python_version=python_version)
+    print(create_msvc_data_files.__doc__)
     if verbose:
         for name in filelist:
-            print "  ", name
-        
-    return [("Microsoft.VC90.CRT", filelist),]
+            print("  ", name)
+    msvc_version = get_msvc_version(python_version)
+    msvc_major = msvc_version.split('.')[0]
+    if msvc_major == '9':
+        return [("Microsoft.VC90.CRT", filelist),]
+    else:
+        return [("", filelist),]
 
 
 def to_include_files(data_files):
@@ -140,20 +249,15 @@ def strip_version(version):
 def remove_dir(dirname):
     """Remove directory *dirname* and all its contents
     Print details about the operation (progress, success/failure)"""
-    print "Removing directory '%s'..." % dirname,
+    print("Removing directory '%s'..." % dirname, end=' ')
     try:
         shutil.rmtree(dirname, ignore_errors=True)
-        print "OK"
+        print("OK")
     except Exception:
-        print "Failed!"
+        print("Failed!")
         traceback.print_exc()
 
 
-def remove_at_exit(fname):
-    """Remove temporary file *fname* at exit"""
-    atexit.register(os.remove, fname)
-
-
 class Distribution(object):
     """Distribution object
     
@@ -165,7 +269,7 @@ class Distribution(object):
                         'pywin.debugger', 'pywin.debugger.dbgcon',
                         'matplotlib']
     DEFAULT_INCLUDES = []
-    DEFAULT_BIN_EXCLUDES = ['MSVCP90.dll', 'w9xpopen.exe',
+    DEFAULT_BIN_EXCLUDES = ['MSVCP100.dll', 'MSVCP90.dll', 'w9xpopen.exe',
                             'MSVCP80.dll', 'MSVCR80.dll']
     DEFAULT_BIN_INCLUDES = []
     DEFAULT_BIN_PATH_INCLUDES = []
@@ -185,9 +289,10 @@ class Distribution(object):
         self.bin_excludes = self.DEFAULT_BIN_EXCLUDES
         self.bin_path_includes = self.DEFAULT_BIN_PATH_INCLUDES
         self.bin_path_excludes = self.DEFAULT_BIN_PATH_EXCLUDES
-        self.vs2008 = os.name == 'nt'
+        self.msvc = os.name == 'nt'
         self._py2exe_is_loaded = False
         self._pyqt4_added = False
+        self._pyside_added = False
         # Attributes relative to cx_Freeze:
         self.executables = []
     
@@ -207,7 +312,14 @@ class Distribution(object):
               target_name=None, target_dir=None, icon=None,
               data_files=None, includes=None, excludes=None,
               bin_includes=None, bin_excludes=None,
-              bin_path_includes=None, bin_path_excludes=None, vs2008=None):
+              bin_path_includes=None, bin_path_excludes=None, msvc=None):
+        """Setup distribution object
+        
+        Notes:
+          * bin_path_excludes is specific to cx_Freeze (ignored if it's None)
+          * if msvc is None, it's set to True by default on Windows 
+            platforms, False on non-Windows platforms
+        """
         self.name = name
         self.version = strip_version(version) if os.name == 'nt' else version
         self.description = description
@@ -230,19 +342,24 @@ class Distribution(object):
             self.bin_path_includes += bin_path_includes
         if bin_path_excludes is not None:
             self.bin_path_excludes += bin_path_excludes
-        if self.vs2008 is not None:
-            self.vs2008 = vs2008
-        if self.vs2008:
-            self.data_files += create_vs2008_data_files()
+        if msvc is not None:
+            self.msvc = msvc
+        if self.msvc:
+            try:
+                self.data_files += create_msvc_data_files()
+            except IOError:
+                print("Setting the msvc option to False "\
+                      "will avoid this error", file=sys.stderr)
+                raise
         # cx_Freeze:
         self.add_executable(self.script, self.target_name, icon=self.icon)
 
     def add_text_data_file(self, filename, contents):
         """Create temporary data file *filename* with *contents*
         and add it to *data_files*"""
-        file(filename, 'wb').write(contents)
+        open(filename, 'wb').write(contents)
         self.data_files += [("", (filename, ))]
-        remove_at_exit(filename)
+        _remove_later(filename)
     
     def add_data_file(self, filename, destdir=''):
         self.data_files += [(destdir, (filename, ))]
@@ -264,31 +381,99 @@ class Distribution(object):
         self.add_text_data_file('qt.conf', conf)
         
         # Including plugins (.svg icons support, QtDesigner support, ...)
-        if self.vs2008:
+        if self.msvc:
             vc90man = "Microsoft.VC90.CRT.manifest"
-            shutil.copy(osp.join(sys.prefix, vc90man), vc90man)
-            man = file(vc90man, "r").read().replace('<file name="',
+            pyqt_tmp = 'pyqt_tmp'
+            if osp.isdir(pyqt_tmp):
+                shutil.rmtree(pyqt_tmp)
+            os.mkdir(pyqt_tmp)
+            vc90man_pyqt = osp.join(pyqt_tmp, vc90man)
+            man = open(vc90man, "r").read().replace('<file name="',
                                         '<file name="Microsoft.VC90.CRT\\')
-            file(vc90man, 'w').write(man)
+            open(vc90man_pyqt, 'w').write(man)
         for dirpath, _, filenames in os.walk(osp.join(pyqt_path,
                                                       "plugins")):
             filelist = [osp.join(dirpath, f) for f in filenames
                         if osp.splitext(f)[1] in ('.dll', '.py')]
-            if self.vs2008 and [f for f in filelist
+            if self.msvc and [f for f in filelist
                            if osp.splitext(f)[1] == '.dll']:
                 # Where there is a DLL build with Microsoft Visual C++ 2008,
                 # there must be a manifest file as well...
                 # ...congrats to Microsoft for this great simplification!
-                filelist.append(vc90man)
+                filelist.append(vc90man_pyqt)
             self.data_files.append( (dirpath[len(pyqt_path)+len(os.pathsep):],
                                      filelist) )
-        if self.vs2008:
-            remove_at_exit(vc90man)
+        if self.msvc:
+            atexit.register(remove_dir, pyqt_tmp)
         
         # Including french translation
-        self.data_files.append(('translations',
-                                (osp.join(pyqt_path, "translations",
-                                 "qt_fr.qm"), )))
+        fr_trans = osp.join(pyqt_path, "translations", "qt_fr.qm")
+        if osp.exists(fr_trans):
+            self.data_files.append(('translations', (fr_trans, )))
+
+    def add_pyside(self):
+        """Include module PySide to the distribution"""
+        if self._pyside_added:
+            return
+        self._pyside_added = True
+        
+        self.includes += ['PySide.QtDeclarative', 'PySide.QtHelp',
+                          'PySide.QtMultimedia', 'PySide.QtNetwork',
+                          'PySide.QtOpenGL', 'PySide.QtScript',
+                          'PySide.QtScriptTools', 'PySide.QtSql',
+                          'PySide.QtSvg', 'PySide.QtTest',
+                          'PySide.QtUiTools', 'PySide.QtWebKit',
+                          'PySide.QtXml', 'PySide.QtXmlPatterns']
+        
+        import PySide
+        pyside_path = osp.dirname(PySide.__file__)
+        
+        # Configuring PySide
+        conf = os.linesep.join(["[Paths]", "Prefix = .", "Binaries = ."])
+        self.add_text_data_file('qt.conf', conf)
+        
+        # Including plugins (.svg icons support, QtDesigner support, ...)
+        if self.msvc:
+            vc90man = "Microsoft.VC90.CRT.manifest"
+            os.mkdir('pyside_tmp')
+            vc90man_pyside = osp.join('pyside_tmp', vc90man)
+            man = open(vc90man, "r").read().replace('<file name="',
+                                        '<file name="Microsoft.VC90.CRT\\')
+            open(vc90man_pyside, 'w').write(man)
+        for dirpath, _, filenames in os.walk(osp.join(pyside_path, "plugins")):
+            filelist = [osp.join(dirpath, f) for f in filenames
+                        if osp.splitext(f)[1] in ('.dll', '.py')]
+            if self.msvc and [f for f in filelist
+                              if osp.splitext(f)[1] == '.dll']:
+                # Where there is a DLL build with Microsoft Visual C++ 2008,
+                # there must be a manifest file as well...
+                # ...congrats to Microsoft for this great simplification!
+                filelist.append(vc90man_pyside)
+            self.data_files.append(
+                    (dirpath[len(pyside_path)+len(os.pathsep):], filelist) )
+
+        # Replacing dlls found by cx_Freeze by the real PySide Qt dlls:
+        # (http://qt-project.org/wiki/Packaging_PySide_applications_on_Windows)
+        dlls = [osp.join(pyside_path, fname)
+                for fname in os.listdir(pyside_path)
+                if osp.splitext(fname)[1] == '.dll']
+        self.data_files.append( ('', dlls) )
+
+        if self.msvc:
+            atexit.register(remove_dir, 'pyside_tmp')
+        
+        # Including french translation
+        fr_trans = osp.join(pyside_path, "translations", "qt_fr.qm")
+        if osp.exists(fr_trans):
+            self.data_files.append(('translations', (fr_trans, )))
+    
+    def add_qt_bindings(self):
+        """Include Qt bindings, i.e. PyQt4 or PySide"""
+        try:
+            imp.find_module('PyQt4')
+            self.add_modules('PyQt4')
+        except ImportError:
+            self.add_modules('PySide')
 
     def add_matplotlib(self):
         """Include module Matplotlib to the distribution"""
@@ -308,17 +493,13 @@ class Distribution(object):
                                     '.xpm', '.ppm', '.npy', '.afm', '.ttf'))
 
     def add_modules(self, *module_names):
-        """Include module *module_name*
-        
-        Notes:
-          * bin_path_excludes is specific to cx_Freeze (ignored if it's None)
-          * if vs2008 is None, it's set to True by default on Windows platforms,
-            False on non-Windows platforms
-        """
+        """Include module *module_name*"""
         for module_name in module_names:
-            print "Configuring module '%s'" % module_name
+            print("Configuring module '%s'" % module_name)
             if module_name == 'PyQt4':
                 self.add_pyqt4()
+            elif module_name == 'PySide':
+                self.add_pyside()
             elif module_name == 'scipy.io':
                 self.includes += ['scipy.io.matlab.streams']
             elif module_name == 'matplotlib':
@@ -351,10 +532,22 @@ class Distribution(object):
                         if osp.splitext(fname)[1] == '.py':
                             modname = 'sphinx.ext.%s' % osp.splitext(fname)[0]
                             self.includes.append(modname)
+            elif module_name == 'pygments':
+                self.includes += ['pygments', 'pygments.formatters',
+                                  'pygments.lexers', 'pygments.lexers.agile']
+            elif module_name == 'zmq':
+                # FIXME: this is not working, yet... (missing DLL)
+                self.includes += ['zmq', 'zmq.core._poll', 'zmq.core._version', 'zmq.core.constants', 'zmq.core.context', 'zmq.core.device', 'zmq.core.error', 'zmq.core.message', 'zmq.core.socket', 'zmq.core.stopwatch']
+                if os.name == 'nt':
+                    self.bin_includes += ['libzmq.dll']
             elif module_name == 'guidata':
                 self.add_module_data_files('guidata', ("images", ),
                                        ('.png', '.svg'), copy_to_root=False)
-                self.add_pyqt4()
+                try:
+                    imp.find_module('PyQt4')
+                    self.add_pyqt4()
+                except ImportError:
+                    self.add_pyside()
             elif module_name == 'guiqwt':
                 self.add_module_data_files('guiqwt', ("images", ),
                                        ('.png', '.svg'), copy_to_root=False)
@@ -371,7 +564,8 @@ class Distribution(object):
                     raise RuntimeError("Module not supported: %s" % module_name)
 
     def add_module_data_dir(self, module_name, data_dir_name, extensions,
-                            copy_to_root=True, verbose=False):
+                            copy_to_root=True, verbose=False,
+                            exclude_dirs=[]):
         """
         Collect data files in *data_dir_name* for module *module_name*
         and add them to *data_files*
@@ -381,38 +575,41 @@ class Distribution(object):
         nstrip = len(module_dir) + len(osp.sep)
         data_dir = osp.join(module_dir, data_dir_name)
         if not osp.isdir(data_dir):
-            raise IOError, "Directory not found: %s" % data_dir
+            raise IOError("Directory not found: %s" % data_dir)
         for dirpath, _dirnames, filenames in os.walk(data_dir):
             dirname = dirpath[nstrip:]
+            if osp.basename(dirpath) in exclude_dirs:
+                continue
             if not copy_to_root:
                 dirname = osp.join(module_name, dirname)
             pathlist = [osp.join(dirpath, f) for f in filenames
-                        if osp.splitext(f)[1] in extensions]
+                        if osp.splitext(f)[1].lower() in extensions]
             self.data_files.append( (dirname, pathlist) )
             if verbose:
                 for name in pathlist:
-                    print "  ", name
+                    print("  ", name)
     
     def add_module_data_files(self, module_name, data_dir_names, extensions,
-                              copy_to_root=True, verbose=False):
+                              copy_to_root=True, verbose=False,
+                              exclude_dirs=[]):
         """
         Collect data files for module *module_name* and add them to *data_files*
         *data_dir_names*: list of dirnames, e.g. ('images', )
         *extensions*: list of file extensions, e.g. ('.png', '.svg')
         """
-        print "Adding module '%s' data files in %s (%s)"\
-              % (module_name, ", ".join(data_dir_names), ", ".join(extensions))
+        print("Adding module '%s' data files in %s (%s)"\
+              % (module_name, ", ".join(data_dir_names), ", ".join(extensions)))
         module_dir = get_module_path(module_name)
         for data_dir_name in data_dir_names:
             self.add_module_data_dir(module_name, data_dir_name, extensions,
-                                     copy_to_root, verbose)
+                                     copy_to_root, verbose, exclude_dirs)
         translation_file = osp.join(module_dir, "locale", "fr", "LC_MESSAGES",
                                     "%s.mo" % module_name)
         if osp.isfile(translation_file):
             self.data_files.append((osp.join(module_name, "locale", "fr",
                                          "LC_MESSAGES"), (translation_file, )))
-            print "Adding module '%s' translation file: %s" % (module_name,
-                                               osp.basename(translation_file))
+            print("Adding module '%s' translation file: %s" % (module_name,
+                                               osp.basename(translation_file)))
 
     def build(self, library, cleanup=True, create_archive=None):
         """Build executable with given library.
@@ -435,7 +632,7 @@ class Distribution(object):
             self.build_cx_freeze(cleanup=cleanup,
                                  create_archive=create_archive)
         else:
-            raise RuntimeError, "Unsupported library %s" % library
+            raise RuntimeError("Unsupported library %s" % library)
     
     def __cleanup(self):
         """Remove old build and dist directories"""
